/*
 * $Id: Phi.java,v 1.2 2002/03/17 10:52:42 nie Exp $
 */

package jp.jasminesoft.gcat.scalc;

/** 
 * Phiは、Newton-Raphson式を用いて
 * 求点座標の垂足緯度の近似値を求めるクラスです。
 *
 * 本パッケージは、琉球大学工学部情報工学科 宮城研究室の成果物
 * を、ジャスミンソフトが整理・統合したものです。再利用を快諾
 * して頂いた宮城 隼夫教授以下、宮城研究室のスタッフにこの場を
 * 借りて感謝致します。
 * 
 * @version $Revision: 1.2 $ $Date: 2002/03/17 10:52:42 $
 * @author  Miho Nagata
 * @author  Yoshinori Nie
 */

class Phi extends Pala {
    /**
     * 赤道から緯度までの子午線弧長を求めるクラス
     */ 
    private ArcLength arc = new ArcLength();

    /**
     * 求点座標の垂足緯度
     */ 
    private double phi[] = new double[5];

    /**
     *　原点までの子午線弧長にX/m0を加えた値。 
     */
    private double sxm;

    /**
     * コンストラクタ
     */
    Phi(double gentenB, double setX) {
	//原点緯度までの子午線弧長
	double arc_l = arc.getArcLength(gentenB); 

	this.sxm = arc_l + setX / m0;
    }

    /**
     * phi[i]による子午線曲率半径を求めるメソッド
     */
    protected double getM(double p) {
	double q = 1.0 - Math.pow(e * Math.sin(p),2);
	double M = ra * (1-Math.pow(e,2)) / Math.sqrt(Math.pow(q,3));
	return M;
    }

    /**
     * phiを返すメソッド
     */
    protected double getPhi() {
	//phiを求める
	for (int i = 0; i < 5; i++) {
	    if(i == 0) {
		phi[i] = gentenB;
	    } else if (i > 0) {
		phi[i] = phi[i-1] - 
		    (arc.getArcLength(phi[i-1]) - sxm) / getM(phi[i-1]); 
	    }
	}
	return phi[4];
    }
}
